-- registers.ads

with System;
with STM32F4;                     use STM32F4;
with STM32F4.GPIO;                use STM32F4.GPIO;
with STM32F4.TIMER;               use STM32F4.TIMER;
with STM32F4.Reset_Clock_Control; use STM32F4.Reset_Clock_Control;
with STM32F4.SYSCONFIG_Control;   use STM32F4.SYSCONFIG_Control;
with Ada.Unchecked_Conversion;

package Registers is

   type Bits is array (Natural range <>) of Boolean with Pack;

   subtype Bits_32 is Bits (0 .. 31);
   subtype Bits_16 is Bits (0 .. 15);
   subtype Bits_8 is Bits (0 .. 7);

   type BIT_NUM is range 0..15;

   function W_From_B32 is
     new Ada.Unchecked_Conversion (Source => Bits_32,
                                   Target => Word);

   function B32_From_W is
     new Ada.Unchecked_Conversion (Source => Word,
                                   Target => Bits_32);


   function HW_From_B16 is
     new Ada.Unchecked_Conversion (Source => Bits_16,
                                   Target => HWord);

   function B16_From_HW is
     new Ada.Unchecked_Conversion (Source => HWord,
                                   Target => Bits_16);

   function B_From_B8 is
     new Ada.Unchecked_Conversion (Source => Bits_8,
                                   Target => Byte);

   function B8_From_B is
     new Ada.Unchecked_Conversion (Source => Byte,
                                   Target => Bits_8);

   pragma Warnings (Off, "*may call Last_Chance_Handler");
   pragma Warnings (Off, "*may be incompatible with alignment of object");

   RCC : RCC_Register with
     Volatile,
     Address => System'To_Address (RCC_Base);

   GPIOB : GPIO_Register with
     Volatile,
     Address => System'To_Address (GPIOB_Base);
   pragma Import (Ada, GPIOB);

   GPIOC : GPIO_Register with
     Volatile,
     Address => System'To_Address (GPIOC_Base);
   pragma Import (Ada, GPIOC);

   GPIOD : GPIO_Register with
     Volatile,
     Address => System'To_Address (GPIOD_Base);
   pragma Import (Ada, GPIOD);

   GPIOE : GPIO_Register with
     Volatile,
     Address => System'To_Address (GPIOE_Base);
   pragma Import (Ada, GPIOE);

   EXTI : EXTI_Register with
     Volatile,
     Address => System'To_Address (EXTI_Base);
   pragma Import (Ada, EXTI);

   SYSCFG : SYSCFG_Register with
     Volatile,
     Address => System'To_Address (SYSCFG_Base);
   pragma Import (Ada, SYSCFG);

   TIM6 : TIM_Register with
     Volatile,
     Address => System'To_Address (TIM6_Base);
   pragma Import (Ada, TIM6);



   pragma Warnings (On, "*may call Last_Chance_Handler");
   pragma Warnings (On, "*may be incompatible with alignment of object");

end Registers;
