-- sevenseg.adb

with Last_Chance_Handler;  pragma Unreferenced (Last_Chance_Handler);
with System;
with Ada.Real_Time; use Ada.Real_Time;
with Registers;     use Registers;
with LEDs;          use LEDs;
with MUX_Timer;     use MUX_Timer;
with STM32F4;       use STM32F4;

procedure SevenSeg is
   pragma Priority (System.Priority'First);
   Next_Time : Time := Clock;
   Tenths    : DISP_VAL := 0;
   Per_Sec   : UPDATE_RATE := 100;
   Buttons   : Bits_32;
   Rate      : Boolean := False; -- True when showing rate, False showing Tenths
   Stop      : Boolean := False;
   Last_Stop : Boolean := False;
begin
   On(Green);
   MUX_Timer.Enable;
   loop
      MUX_Timer.Set_Interrupt_Time(Per_Sec);

      Buttons := B32_From_W(not GPIOE.IDR); -- read buttons @ 10..13
      if Buttons(B_CLEAR) then
	 Tenths := 0;
	 Rate := False;  -- showing tenths, not display rate
      end if;

      if Buttons(B_SS) and not Last_Stop then  -- found a leading button edge
	 Stop := not Stop;
	 Rate := False;  -- showing tenths, not display rate
      end if;
      Last_Stop := Buttons(B_SS);
      if not Stop then
	 Tenths := Tenths + 1;
      end if;

      if Buttons(B_SLOW) then  -- slow down
	 Per_Sec := Per_Sec - 1;
	 Rate := True;  -- showing display rate
      end if;

      if Buttons(B_FAST) then  -- speed up
	 Per_Sec := Per_Sec + 1;
	 Rate := True;  -- showing display rate
      end if;

      if Rate then
	 -- pass current time in 1/10 sec units to timer display
	 MUX_Timer.Set_Val(DISP_VAL(Per_Sec), False);
      else
	 -- pass current time in 1/10 sec units to timer display
	 MUX_Timer.Set_Val(Tenths, True);
      end if;

     -- calculate next 100ms clock point and sleep until then
      Next_Time := Next_Time + Milliseconds(100);
      delay until Next_Time;
   end loop;
end SevenSeg;
